[{"locale":"es"},{"category":"Lu","mappings":{"default":{"default":"negrita mayúscula A"}},"key":"1D400"},{"category":"Lu","mappings":{"default":{"default":"negrita mayúscula B"}},"key":"1D401"},{"category":"Lu","mappings":{"default":{"default":"negrita mayúscula C"}},"key":"1D402"},{"category":"Lu","mappings":{"default":{"default":"negrita mayúscula D"}},"key":"1D403"},{"category":"Lu","mappings":{"default":{"default":"negrita mayúscula E"}},"key":"1D404"},{"category":"Lu","mappings":{"default":{"default":"negrita mayúscula F"}},"key":"1D405"},{"category":"Lu","mappings":{"default":{"default":"negrita mayúscula G"}},"key":"1D406"},{"category":"Lu","mappings":{"default":{"default":"negrita mayúscula H"}},"key":"1D407"},{"category":"Lu","mappings":{"default":{"default":"negrita mayúscula I"}},"key":"1D408"},{"category":"Lu","mappings":{"default":{"default":"negrita mayúscula J"}},"key":"1D409"},{"category":"Lu","mappings":{"default":{"default":"negrita mayúscula K"}},"key":"1D40A"},{"category":"Lu","mappings":{"default":{"default":"negrita mayúscula L"}},"key":"1D40B"},{"category":"Lu","mappings":{"default":{"default":"negrita mayúscula M"}},"key":"1D40C"},{"category":"Lu","mappings":{"default":{"default":"negrita mayúscula N"}},"key":"1D40D"},{"category":"Lu","mappings":{"default":{"default":"negrita mayúscula O"}},"key":"1D40E"},{"category":"Lu","mappings":{"default":{"default":"negrita mayúscula P"}},"key":"1D40F"},{"category":"Lu","mappings":{"default":{"default":"negrita mayúscula Q"}},"key":"1D410"},{"category":"Lu","mappings":{"default":{"default":"negrita mayúscula R"}},"key":"1D411"},{"category":"Lu","mappings":{"default":{"default":"negrita mayúscula S"}},"key":"1D412"},{"category":"Lu","mappings":{"default":{"default":"negrita mayúscula T"}},"key":"1D413"},{"category":"Lu","mappings":{"default":{"default":"negrita mayúscula U"}},"key":"1D414"},{"category":"Lu","mappings":{"default":{"default":"negrita mayúscula V"}},"key":"1D415"},{"category":"Lu","mappings":{"default":{"default":"negrita mayúscula W"}},"key":"1D416"},{"category":"Lu","mappings":{"default":{"default":"negrita mayúscula X"}},"key":"1D417"},{"category":"Lu","mappings":{"default":{"default":"negrita mayúscula Y"}},"key":"1D418"},{"category":"Lu","mappings":{"default":{"default":"negrita mayúscula Z"}},"key":"1D419"},{"category":"Ll","mappings":{"default":{"default":"negrita a"}},"key":"1D41A"},{"category":"Ll","mappings":{"default":{"default":"negrita b"}},"key":"1D41B"},{"category":"Ll","mappings":{"default":{"default":"negrita c"}},"key":"1D41C"},{"category":"Ll","mappings":{"default":{"default":"negrita d"}},"key":"1D41D"},{"category":"Ll","mappings":{"default":{"default":"negrita e"}},"key":"1D41E"},{"category":"Ll","mappings":{"default":{"default":"negrita f"}},"key":"1D41F"},{"category":"Ll","mappings":{"default":{"default":"negrita g"}},"key":"1D420"},{"category":"Ll","mappings":{"default":{"default":"negrita h"}},"key":"1D421"},{"category":"Ll","mappings":{"default":{"default":"negrita i"}},"key":"1D422"},{"category":"Ll","mappings":{"default":{"default":"negrita j"}},"key":"1D423"},{"category":"Ll","mappings":{"default":{"default":"negrita k"}},"key":"1D424"},{"category":"Ll","mappings":{"default":{"default":"negrita l"}},"key":"1D425"},{"category":"Ll","mappings":{"default":{"default":"negrita m"}},"key":"1D426"},{"category":"Ll","mappings":{"default":{"default":"negrita n"}},"key":"1D427"},{"category":"Ll","mappings":{"default":{"default":"negrita o"}},"key":"1D428"},{"category":"Ll","mappings":{"default":{"default":"negrita p"}},"key":"1D429"},{"category":"Ll","mappings":{"default":{"default":"negrita q"}},"key":"1D42A"},{"category":"Ll","mappings":{"default":{"default":"negrita r"}},"key":"1D42B"},{"category":"Ll","mappings":{"default":{"default":"negrita s"}},"key":"1D42C"},{"category":"Ll","mappings":{"default":{"default":"negrita t"}},"key":"1D42D"},{"category":"Ll","mappings":{"default":{"default":"negrita u"}},"key":"1D42E"},{"category":"Ll","mappings":{"default":{"default":"negrita v"}},"key":"1D42F"},{"category":"Ll","mappings":{"default":{"default":"negrita w"}},"key":"1D430"},{"category":"Ll","mappings":{"default":{"default":"negrita x"}},"key":"1D431"},{"category":"Ll","mappings":{"default":{"default":"negrita y"}},"key":"1D432"},{"category":"Ll","mappings":{"default":{"default":"negrita z"}},"key":"1D433"}]
